@@ -9,10 +9,11 @@
 """
 import numpy as np
 import thermosteam as tmo
+from thermosteam import functional as fn
 from . import indexer
 from . import equilibrium as eq
 from . import units_of_measure as thermo_units
-from .exceptions import DimensionError
+from .exceptions import DimensionError, InfeasibleRegion
 from chemicals.elements import array_to_atoms, symbol_to_index
 from . import utils
 
@@ -48,23 +49,25 @@ class Stream:
 
     Parameters
     ----------
-    ID='' : str
+    ID : str, optional
         A unique identification. If ID is None, stream will not be registered.
         If no ID is given, stream will be registered with a unique ID.
-    flow=() : Iterable[float]
+    flow : Iterable[float], optional
         All flow rates corresponding to chemical `IDs`.
-    phase='l' : 'l', 'g', or 's'
-        Either gas (g), liquid (l), or solid (s).
-    T=298.15 : float
-        Temperature [K].
-    P=101325 : float
-        Pressure [Pa].
-    units='kmol/hr' : str
+    phase : 'l', 'g', or 's'
+        Either gas (g), liquid (l), or solid (s). Defaults to 'l'.
+    T : float
+        Temperature [K]. Defaults to 298.15.
+    P : float
+        Pressure [Pa]. Defaults to 101325.
+    units : str, optional
         Flow rate units of measure (only mass, molar, and
-        volumetric flow rates are valid).
-    price=0 : float
-        Price per unit mass [USD/kg].
-    thermo=None : :class:`~thermosteam.Thermo`
+        volumetric flow rates are valid). Defaults to 'kmol/hr'.
+    price : float, optional
+        Price per unit mass [USD/kg]. Defaults to 0.
+    total_flow : float, optional
+        Total flow rate.
+    thermo : :class:`~thermosteam.Thermo`, optional
         Thermo object to initialize input and output streams. Defaults to
         `biosteam.settings.get_thermo()`.
     **chemical_flows : float
@@ -244,17 +247,23 @@ class Stream:
     _flow_cache = {}
 
     def __init__(self, ID= '', flow=(), phase='l', T=298.15, P=101325.,
-                 units='kmol/hr', price=0., thermo=None, **chemical_flows):
+                 units=None, price=0., total_flow=None, thermo=None, 
+                 **chemical_flows):
         self._thermal_condition = tmo.ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
         self._init_indexer(flow, phase, thermo.chemicals, chemical_flows)
         self.price = price
-        if units != 'kmol/hr':
+        mol = self.mol
+        if units:
             name, factor = self._get_flow_name_and_factor(units)
             flow = getattr(self, name)
-            flow[:] = self.mol / factor
+            material_data = mol / factor
+            if total_flow: material_data *= total_flow / material_data.sum()
+            flow[:] = material_data
+        elif total_flow:
+            mol *= total_flow / mol.sum()
         self._sink = self._source = None # For BioSTEAM
-        self._init_cache()
+        self.reset_cache()
         self._register(ID)
         self._link = None
 
@@ -379,6 +388,30 @@ class Stream:
         """
         return self._imol.isempty()
 
+    def sanity_check(self):
+        """
+        Raise an InfeasibleRegion error if flow rates are infeasible.
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water'], cache=True)
+        >>> s1 = tmo.Stream('s1')
+        >>> s1.sanity_check()
+        >>> s1.mol[0] = -1.
+        >>> s1.sanity_check()
+        Traceback (most recent call last):
+        InfeasibleRegion: stream 's1' has negative material flow rates
+        
+        """
+        material = self._imol._data
+        if fn.infeasible(material):
+            region = 'negative material flow rates'
+            msg = f"stream '{self}' has " + region
+            raise InfeasibleRegion(region, msg)
+        else:
+            material[material < 0.] = 0. 
+
     @property
     def vapor_fraction(self):
         """Molar vapor fraction."""
@@ -436,7 +469,8 @@ class Stream:
                     np.asarray(flow, dtype=float), phase, chemicals)
         self._imol = imol
 
-    def _init_cache(self):
+    def reset_cache(self):
+        """Reset cache regarding equilibrium methods."""
         self._bubble_point_cache = eq.BubblePointCache()
         self._dew_point_cache = eq.DewPointCache()
 
@@ -1148,7 +1182,7 @@ class Stream:
         imol._data = imol._data.copy()
         imol._phase = imol._phase.copy()
         self._thermal_condition = self._thermal_condition.copy()
-        self._init_cache()
+        self.reset_cache()
         self._link = None
     
     def copy_like(self, other):
@@ -1338,7 +1372,7 @@ class Stream:
         new._thermo = self._thermo
         new._imol = self._imol.copy()
         new._thermal_condition = self._thermal_condition.copy()
-        new._init_cache()
+        new.reset_cache()
         new.price = 0
         new.ID = ID
         return new
@@ -1366,7 +1400,7 @@ class Stream:
         new._imol = imol = self._imol._copy_without_data()
         imol._data = self._imol._data
         new._thermal_condition = self._thermal_condition.copy()
-        new._init_cache()
+        new.reset_cache()
         new._link = self
         return new
     
