@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -9,6 +9,7 @@
 """
 from numpy import asarray, array
 import flexsolve as flx
+from .domain import vle_domain
 from ..exceptions import InfeasibleRegion, DomainError
 from .solve_vle_composition import solve_y
 from .. import functional as fn
@@ -66,7 +67,6 @@ class BubblePoint:
     """
     __slots__ = ('chemicals', 'IDs', 'gamma', 'phi', 'pcf',
                  'P', 'T', 'y', 'Psats', 'Tmin', 'Tmax', 'Pmin', 'Pmax')
-    Tmin_default = 150.
     _cached = {}
     def __init__(self, chemicals=(), thermo=None):
         thermo = settings.get_default_thermo(thermo)
@@ -82,8 +82,9 @@ class BubblePoint:
             self.phi = thermo.Phi(chemicals)
             self.pcf = thermo.PCF(chemicals)
             self.Psats = Psats = [i.Psat for i in chemicals]
-            self.Tmin = Tmin = max(max([i.Tmin for i in Psats]) + 1e-3, self.Tmin_default)
-            self.Tmax = Tmax = min([i.Tmax for i in Psats]) - 1e-3
+            Tmin, Tmax = vle_domain(chemicals)
+            self.Tmin = Tmin
+            self.Tmax = Tmax
             self.Pmin = min([i(Tmin) for i in Psats])
             self.Pmax = max([i(Tmax) for i in Psats])
             self.chemicals = chemicals
@@ -163,12 +164,14 @@ class BubblePoint:
         (353.7543, array([0.381, 0.619]))
         
         """
+        if P > self.Pmax: P = self.Pmax
+        elif P < self.Pmin: P = self.Pmin
         self.P = P
         f = self._T_error
         z_norm = z / z.sum()
-        z_over_P = z_norm/P
+        z_over_P = z/P
         args = (P, z_over_P, z_norm)
-        T_guess = self.T or self._T_ideal(z_over_P) 
+        T_guess = self._T_ideal(z_over_P) 
         try:
             T = flx.aitken_secant(f, T_guess, T_guess + 1e-3,
                                   1e-9, 5e-12, args,
@@ -211,13 +214,15 @@ class BubblePoint:
         (91830.9798, array([0.419, 0.581]))
         
         """
+        if T > self.Tmax: T = self.Tmax
+        elif T < self.Tmin: T = self.Tmin
         self.T = T
         Psat = array([i(T) for i in self.Psats])
         z_norm = z / z.sum()
         z_Psat_gamma_pcf = z * Psat * self.gamma(z_norm, T) * self.pcf(z_norm, T)
         f = self._P_error
         args = (T, z_Psat_gamma_pcf)
-        P_guess = self.P or self._P_ideal(z_Psat_gamma_pcf)
+        P_guess = self._P_ideal(z_Psat_gamma_pcf)
         try:
             P = flx.aitken_secant(f, P_guess, P_guess-1, 1e-3, 1e-9,
                                   args, checkiter=False)
