@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -20,12 +20,12 @@ def horner(T, coeffs):
     return tot 
 
 @njitable(cache=True)
-def normalize(array, minimum=1e-16):
+def normalize(array, sum_array=None, minimum=1e-16):
     """
     Return a normalized array to a magnitude of 1.
     If magnitude is zero, all fractions will have equal value.
     """
-    sum_array = array.sum()
+    if sum_array is None: sum_array = array.sum()
     if sum_array < minimum:
         size = array.size
         return np.ones(size)/size
@@ -143,4 +143,19 @@ def rho_to_V(rho, MW):
     '''
     return MW/rho/1000.
 
+@njitable(cache=True)
+def remove_negligible_negative_values(material: np.ndarray):
+    material_sum = np.abs(material).sum()
+    negative = material < 0.
+    if material_sum > 1e-16:
+        negligible = material / material_sum > -1e-16
+        material[negative & negligible] = 0. 
+    else:
+        material[negative] = 0. 
+
+@njitable(cache=True)
+def infeasible(material: np.ndarray):
+    material_sum = np.abs(material).sum()
+    return material_sum > 1e-16 and (material / material_sum < -1e-16).any()
+
 del njitable
\ No newline at end of file
