@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -31,6 +31,11 @@ __all__ = ('ChemicalIndexer',
 isa = isinstance
 _new = object.__new__
 
+def raise_material_indexer_index_error():
+    raise IndexError("index by [phase, IDs] where phase is a "
+                     "(str, ellipsis, or missing), and IDs is a "
+                     "(str, tuple(str), ellipisis, or missing)")
+
 def find_main_phase(indexers, default):
     main_indexer, *indexers = indexers
     try:
@@ -60,6 +65,12 @@ class Indexer:
     __slots__ = ('_data',)
     units = None
     
+    def empty(self):
+        self._data[:] = 0
+    
+    def isempty(self):
+        return (self._data == 0.).all()
+    
     def copy(self):
         new = self._copy_without_data()
         new._data = self._data.copy()
@@ -153,9 +164,6 @@ class ChemicalIndexer(Indexer):
     def get_index(self):
         return self._chemicals.get_index
     
-    def empty(self):
-        self._data[:] = 0.
-    
     def mix_from(self, others):
         self.phase = find_main_phase(others, self.phase)
         chemicals = self.chemicals
@@ -224,6 +232,11 @@ class ChemicalIndexer(Indexer):
     def phase(self, phase):
         self._phase.phase = phase
     
+    def get_phase_and_composition(self):
+        """Return phase and composition."""
+        data = self._data
+        return self.phase, data / data.sum()
+    
     def __format__(self, tabs=""):
         if not tabs: tabs = 1
         tabs = int(tabs) 
@@ -327,12 +340,16 @@ class MaterialIndexer(Indexer):
     def __reduce__(self):
         return self.from_data, (self._data, self._phases, self._chemicals, False)
     
+    def phases_are_empty(self, phases):
+        get_phase_index = self.get_phase_index
+        data = self._data
+        for phase in set(self._phases).intersection(phases):
+            if data[get_phase_index(phase)].any(): return False
+        return True
+    
     def sum_across_phases(self):
         return self._data.sum(0)
     
-    def empty(self):
-        self._data[:] = 0
-    
     def copy_like(self, other):
         if isa(other, ChemicalIndexer):
             self.empty()
@@ -493,8 +510,8 @@ class MaterialIndexer(Indexer):
         except KeyError:
             try:
                 index = self._chemicals.get_index(key)
-            except UndefinedChemical:
-                index = self._get_index(key)
+            except UndefinedChemical as error:
+                index = self._get_index(key, error)
             else:
                 index = ChemicalIndex(index)
             cache[key] = index
@@ -503,32 +520,35 @@ class MaterialIndexer(Indexer):
             raise TypeError("only strings, tuples, and ellipsis are valid index keys")
         return index
     
-    def _get_index(self, phase_IDs):
+    def _get_index(self, phase_IDs, undefined_chemical_error):
         if isa(phase_IDs, str):
-            index = self.get_phase_index(phase_IDs)
+            if len(phase_IDs) == 1: 
+                index = self.get_phase_index(phase_IDs)
+            else:
+                raise undefined_chemical_error
         elif phase_IDs is ...:
-            index = slice(None) 
+            index = slice(None)
         else:
+            phase = phase_IDs[0]
+            if isa(phase, str):
+                if len(phase) == 1:
+                    phase_index = self.get_phase_index(phase)
+                else:
+                    raise undefined_chemical_error
+            elif phase is ...:
+                phase_index = slice(None)
+            else:
+                raise_material_indexer_index_error()
             try:
                 phase, IDs = phase_IDs
             except:
-                raise IndexError("index by [phase, IDs] where phase is a "
-                                 "(str, ellipsis, or missing), and IDs is a "
-                                 "(str, tuple(str), ellipisis, or missing)")
+                raise_material_indexer_index_error()
             if isa(IDs, (str, tuple)):
-                IDs_index = self._chemicals.get_index(IDs)
+                index = (phase_index, self._chemicals.get_index(IDs))
             elif IDs is ...:
-                IDs_index = IDs
+                index = (phase_index, IDs)
             else:
                 raise TypeError("only strings, tuples, and ellipsis are valid index keys")
-            if isa(phase, str):
-                index = (self.get_phase_index(phase), IDs_index)
-            elif phase is ...:
-                index = (phase, IDs_index)
-            else:
-                raise IndexError("index by [phase, IDs] where phase is a "
-                                 "(str, ellipsis, or missing), and IDs is a "
-                                 "(str, tuple(str), ellipisis, or missing)")
         return index
     
     def __iter__(self):
