@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -9,12 +9,13 @@
 This module contains functions for modeling separations in unit operations.
 
 """
+from warnings import warn
 import thermosteam as tmo
 import flexsolve as flx
 import numpy as np
 from .utils import thermo_user
 from .exceptions import InfeasibleRegion
-from .equilibrium import phase_fraction
+from .equilibrium import phase_fraction as compute_phase_fraction
 
 __all__ = (
     'split', 'mix_and_split',
@@ -29,9 +30,31 @@ __all__ = (
     'MultiStageLLE',
     'single_component_flow_rates_for_multi_stage_lle_without_side_draws',
     'flow_rates_for_multi_stage_extration_without_side_draws',
-    'chemical_splits'
+    'chemical_splits',
+    'phase_fraction',
 )
 
+def check_partition_infeasibility(infeasible_index, strict, stacklevel=1):
+    if infeasible_index.any():
+        if strict:
+            raise InfeasibleRegion('negative flow rates in equilibrium '
+                                   'solution; partition data')
+        else:
+            warning = RuntimeWarning(
+                'phase equilibrium solution results in negative flow rates; '
+                'negative flows have been removed from solution'
+            )
+            warn(warning, stacklevel=stacklevel+1)
+
+
+def handle_infeasible_flow_rates(mol, maxmol, strict, stacklevel=1):
+    infeasible_index = mol < 0.
+    check_partition_infeasibility(infeasible_index, strict, stacklevel+1)
+    mol[infeasible_index] = 0.
+    infeasible_index = mol > maxmol
+    check_partition_infeasibility(infeasible_index, strict, stacklevel+1)
+    mol[infeasible_index] = maxmol[infeasible_index]
+
 
 # %% Mixing, splitting, and moisture content
 
@@ -296,7 +319,10 @@ def partition_coefficients(IDs, top, bottom):
     array([0.629, 1.59 ])
 
     """
-    return top.get_normalized_mol(IDs) / bottom.get_normalized_mol(IDs)
+    numerator = top.get_normalized_mol(IDs)
+    denominator = bottom.get_normalized_mol(IDs)
+    denominator[denominator < 1e-24] = 1e-24
+    return numerator / denominator
 
 def chemical_splits(a, b=None, mixed=None):
     """
@@ -392,7 +418,80 @@ def lle_partition_coefficients(top, bottom):
     IDs = tuple([i.ID for i in bottom.lle_chemicals])
     return IDs, partition_coefficients(IDs, top, bottom)
 
-def partition(feed, top, bottom, IDs, K, phi=None):
+def phase_fraction(feed, IDs, K, phi=None, top_chemicals=None, 
+                   bottom_chemicals=None, strict=False, stacklevel=1):
+    """
+    Return the phase fraction given a stream and partition coeffiecients.
+
+    Parameters
+    ----------
+    feed : Stream
+        Mixed feed.
+    IDs : tuple[str]
+        IDs of chemicals in equilibrium.
+    K : 1d array
+        Partition coefficeints corresponding to IDs.
+    phi : float, optional
+        Guess phase fraction in top phase.
+    top_chemicals : tuple[str], optional
+        Chemicals that remain in the top fluid.
+    bottom_chemicals : tuple[str], optional
+        Chemicals that remain in the bottom fluid.
+    strict : bool, optional
+        Whether to raise an InfeasibleRegion exception when solution results
+        in negative flow rates or to remove negative flows and issue a warning. 
+        Defaults to False.
+
+    Returns
+    -------
+    phi : float
+        Phase fraction in top phase.
+
+    Notes
+    -----
+    Chemicals not in equilibrium end up in the top phase.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import thermosteam as tmo
+    >>> tmo.settings.set_thermo(['Water', 'Ethanol', tmo.Chemical('NaCl', default=True), 
+    ...                          tmo.Chemical('O2', phase='g')], cache=True)
+    >>> IDs = ('Water', 'Ethanol')
+    >>> K = np.array([0.629, 1.59])
+    >>> feed = tmo.Stream('feed', Water=20, Ethanol=20, O2=0.1)
+    >>> tmo.separations.phase_fraction(feed, IDs, K)
+    0.500
+    >>> feed = tmo.Stream('feed', Water=20, Ethanol=20, NaCl=0.1, O2=0.1)
+    >>> tmo.separations.phase_fraction(feed, IDs, K, 
+    ...                                top_chemicals=('O2',),
+    ...                                bottom_chemicals=('NaCl'))
+    0.500
+    
+    """
+    mol = feed.imol[IDs]
+    F_mol = mol.sum()
+    Fa = feed.imol[top_chemicals].sum() if top_chemicals else 0.
+    if bottom_chemicals:
+        bottom_flows = feed.imol[bottom_chemicals]
+        Fb = bottom_flows.sum()
+    else:
+        Fb = 0.
+    F_mol += Fa + Fb
+    z_mol = mol / F_mol
+    phi = compute_phase_fraction(z_mol, K, phi, Fa/F_mol, Fb/F_mol)
+    if phi <= 0.:
+        phi = 0.
+    elif phi < 1.:
+        x = z_mol / (phi * K + (1. - phi))
+        bottom_mol = x * (1. - phi) * F_mol
+        handle_infeasible_flow_rates(bottom_mol, mol, strict, stacklevel+1)
+    else:
+        phi = 1.
+    return phi
+
+def partition(feed, top, bottom, IDs, K, phi=None, top_chemicals=None, 
+              bottom_chemicals=None, strict=False, stacklevel=1):
     """
     Run equilibrium of feed to top and bottom streams given partition 
     coeffiecients and return the phase fraction.
@@ -411,6 +510,14 @@ def partition(feed, top, bottom, IDs, K, phi=None):
         Partition coefficeints corresponding to IDs.
     phi : float, optional
         Guess phase fraction in top phase.
+    top_chemicals : tuple[str], optional
+        Chemicals that remain in the top fluid.
+    bottom_chemicals : tuple[str], optional
+        Chemicals that remain in the bottom fluid.
+    strict : bool, optional
+        Whether to raise an InfeasibleRegion exception when solution results
+        in negative flow rates or to remove negative flows and issue a warning. 
+        Defaults to False.
 
     Returns
     -------
@@ -425,14 +532,15 @@ def partition(feed, top, bottom, IDs, K, phi=None):
     --------
     >>> import numpy as np
     >>> import thermosteam as tmo
-    >>> tmo.settings.set_thermo(['Water', 'Ethanol', tmo.Chemical('O2', phase='g')], cache=True)
+    >>> tmo.settings.set_thermo(['Water', 'Ethanol', tmo.Chemical('NaCl', default=True), 
+    ...                          tmo.Chemical('O2', phase='g')], cache=True)
     >>> IDs = ('Water', 'Ethanol')
     >>> K = np.array([0.629, 1.59])
     >>> feed = tmo.Stream('feed', Water=20, Ethanol=20, O2=0.1)
     >>> top = tmo.Stream('top')
     >>> bottom = tmo.Stream('bottom')
     >>> tmo.separations.partition(feed, top, bottom, IDs, K)
-    0.5002512677600628
+    0.500
     >>> top.show()
     Stream: top
      phase: 'l', T: 298.15 K, P: 101325 Pa
@@ -444,16 +552,48 @@ def partition(feed, top, bottom, IDs, K, phi=None):
      phase: 'l', T: 298.15 K, P: 101325 Pa
      flow (kmol/hr): Water    12.3
                      Ethanol  7.72
+    >>> feed = tmo.Stream('feed', Water=20, Ethanol=20, NaCl=0.1, O2=0.1)
+    >>> tmo.separations.partition(feed, top, bottom, IDs, K, 
+    ...                           top_chemicals=('O2',),
+    ...                           bottom_chemicals=('NaCl'))
+    0.500
+    >>> top.show()
+    Stream: top
+     phase: 'l', T: 298.15 K, P: 101325 Pa
+     flow (kmol/hr): Water    7.73
+                     Ethanol  12.3
+                     O2       0.1
+    >>> bottom.show()
+    Stream: bottom
+     phase: 'l', T: 298.15 K, P: 101325 Pa
+     flow (kmol/hr): Water    12.3
+                     Ethanol  7.72
+                     NaCl     0.1
 
     """
     feed_mol = feed.mol
     mol = feed.imol[IDs]
     F_mol = mol.sum()
+    if not bottom.shares_flow_rate_with(feed): bottom.empty()
+    Fa = feed.imol[top_chemicals].sum() if top_chemicals else 0.
+    if bottom_chemicals:
+        bottom.imol[bottom_chemicals] = bottom_flows = feed.imol[bottom_chemicals]
+        Fb = bottom_flows.sum()
+    else:
+        Fb = 0.
+    F_mol += Fa + Fb
     z_mol = mol / F_mol
-    phi = phase_fraction(z_mol, K, phi)
-    x = z_mol / (phi * K + (1 - phi))
-    bottom.empty()
-    bottom.imol[IDs] = x * (1 - phi) * F_mol
+    phi = compute_phase_fraction(z_mol, K, phi, Fa/F_mol, Fb/F_mol)
+    if phi <= 0.:
+        bottom.imol[IDs] = mol
+        phi = 0.
+    elif phi < 1.:
+        x = z_mol / (phi * K + (1. - phi))
+        bottom_mol = x * (1. - phi) * F_mol
+        handle_infeasible_flow_rates(bottom_mol, mol, strict, stacklevel+1)
+        bottom.imol[IDs] = bottom_mol
+    else:
+        phi = 1.        
     top.mol[:] = feed_mol - bottom.mol
     return phi
 
@@ -765,6 +905,8 @@ class StageLLE:
     __slots__ = ('feed', 'raffinate', 'solvent', 'extract', 'multi_stream',
                  'carrier_chemical', '_thermo', '_phi', '_K', '_IDs')
     
+    strict_infeasibility_check = False
+    
     def __init__(self, T=298.15, P=101325, feed=None,
                  solvent=None, thermo=None, carrier_chemical=None):
         self.feed = feed
@@ -776,6 +918,7 @@ class StageLLE:
         self.raffinate = multi_stream['l']
         self.extract = multi_stream['L']
         self.carrier_chemical = carrier_chemical
+        self._phi = None
         
     @property
     def T(self):
@@ -791,27 +934,32 @@ class StageLLE:
     def P(self, P):
         self.multi_stream.P = P
         
-    def partition(self, partition_data=None):
+    def partition(self, partition_data=None, update=True, stacklevel=1):
         multi_stream = self.multi_stream
-        multi_stream.mix_from([self.feed, self.solvent])
-        lle = multi_stream.lle
+        if not update: data = multi_stream.get_data()
+        multi_stream.mix_from([self.feed, self.solvent], energy_balance=False)
         if partition_data:
-            top = multi_stream['l']
-            bottom = multi_stream['L']
-            lle = multi_stream.lle
-            phi = partition_data['phi']
+            raffinate = multi_stream['l']
+            extract = multi_stream['L']
             self._K = K = partition_data['K']
             self._IDs = IDs = partition_data['IDs']
-            self._phi = partition(multi_stream, top, bottom, IDs, K, phi)
+            args = (IDs, K, self._phi or partition_data['phi'], 
+                    partition_data.get('raffinate_chemicals'),
+                    partition_data.get('extract_chemicals'),
+                    self.strict_infeasibility_check, stacklevel+1)
+            self._phi = partition(multi_stream, raffinate, extract, *args)
         else:
+            lle = multi_stream.lle                
             lle(self.T, top_chemical=self.carrier_chemical or self.feed.main_chemical)
             self._IDs = tuple([i.ID for i in lle._lle_chemicals])
             self._phi = lle._phi
-            self._K = lle._K 
+            self._K = lle._K
+        if not update: multi_stream.set_data(data)
         
     def balance_raffinate_flows(self):
         total_mol = self.feed.mol + self.solvent.mol
         extract_mol = self.extract.mol
+        handle_infeasible_flow_rates(extract_mol, total_mol, self.strict_infeasibility_check, 2)
         self.raffinate.mol[:] = total_mol - extract_mol
         
     @property
@@ -822,7 +970,7 @@ class StageLLE:
         return self._phi
     @property
     def K(self):
-        return self._K 
+        return self._K
     
     def __repr__(self):
         return f"{type(self).__name__}(T={self.T}, P={self.P})"
@@ -864,7 +1012,7 @@ class MultiStageLLE:
     Stream: 
      phase: 'l', T: 298.15 K, P: 101325 Pa
      flow (kmol/hr): Water     413
-                     Methanol  8.4
+                     Methanol  8.5
                      Octanol   0.1
     >>> stages.extract.show()
     Stream: 
@@ -892,8 +1040,8 @@ class MultiStageLLE:
     Stream: 
      phase: 'l', T: 298.15 K, P: 101325 Pa
      flow (kmol/hr): Water     4.1e+03
-                     Methanol  1.2
-                     Octanol   1.5
+                     Methanol  1.3
+                     Octanol   1.1
     >>> stages.extract.show()
     Stream: 
      phase: 'L', T: 298.15 K, P: 101325 Pa
@@ -901,6 +1049,30 @@ class MultiStageLLE:
                      Methanol  499
                      Octanol   5e+03
     
+    Because octanol and water do not mix well, it may be a good idea to assume
+    that these solvents do not mix at all:
+        
+    >>> N_stages = 20
+    >>> stages = tmo.separations.MultiStageLLE(N_stages, feed, solvent,
+    ...     partition_data={
+    ...         'K': np.array([0.7244]),
+    ...         'IDs': ('Methanol',),
+    ...         'raffinate_chemicals': ('Water',),
+    ...         'extract_chemicals': ('Octanol',),
+    ...     }
+    ... )
+    >>> stages.simulate_multi_stage_lle_without_side_draws()
+    >>> stages.raffinate.show()
+    Stream: 
+     phase: 'l', T: 298.15 K, P: 101325 Pa
+     flow (kmol/hr): Water     5e+03
+                     Methanol  0.199
+    >>> stages.extract.show()
+    Stream: 
+     phase: 'L', T: 298.15 K, P: 101325 Pa
+     flow (kmol/hr): Methanol  500
+                     Octanol   5e+03
+        
     """
     __slots__ = ('stages', 'index', 'multi_stream', 
                  'carrier_chemical', 'extract_flow_rates', 
@@ -942,37 +1114,45 @@ class MultiStageLLE:
         return self.stages[-1].raffinate
     
     def update_multi_stage_lle_without_side_draws(self, extract_flow_rates):
-        if (extract_flow_rates < 0.).any(): raise RuntimeError('negative flow rate')
         index = self.index
         for stage, extract_flow in zip(self.stages, extract_flow_rates):
             stage.extract.mol[index] = extract_flow
-            stage.balance_raffinate_flows()
-    
+            
     def simulate_multi_stage_lle_without_side_draws(self):
         f = self.multi_stage_lle_without_side_draws_iter
-        if hasattr(self, 'extract_flow_rates'):
-            extract_flow_rates = self.extract_flow_rates
-        else:
-            extract_flow_rates = self.initialize_multi_stage_lle_without_side_draws()
-        extract_flow_rates = flx.wegstein(f, extract_flow_rates, xtol=0.1, maxiter=10, checkiter=False)
-        self.extract_flow_rates = extract_flow_rates
+        extract_flow_rates = self.initialize_multi_stage_lle_without_side_draws()
+        self.extract_flow_rates = extract_flow_rates = flx.wegstein(f, extract_flow_rates, xtol=0.1, maxiter=10, checkiter=False)
         self.update_multi_stage_lle_without_side_draws(extract_flow_rates)
+        for i in self.stages: i.balance_raffinate_flows()
     
     def initialize_multi_stage_lle_without_side_draws(self):
         feed = self.feed
         solvent = self.solvent
         multi_stream = self.multi_stream
-        multi_stream.mix_from([feed, solvent]) 
+        multi_stream.mix_from([feed, solvent])
         stages = self.stages
         N_stages = len(stages)
-        if self.partition_data:
+        T = multi_stream.T
+        for i in stages: 
+            i.multi_stream.T = T
+            i.raffinate.empty()
+        if self.partition_data: 
             data = self.partition_data
-            top = multi_stream['l']
-            bottom = multi_stream['L']
+            raffinate = multi_stream['l']
+            extract = multi_stream['L']
             IDs = data['IDs']
             K = data['K']
-            phi = data['phi']
-            data['phi'] = phi = partition(multi_stream, top, bottom, IDs, K, phi)
+            phi = data.get('phi') or feed.F_mol / multi_stream.F_mol
+            raffinate_chemicals = data.get('raffinate_chemicals')
+            extract_chemicals = data.get('extract_chemicals')
+            data['phi'] = phi = partition(multi_stream, raffinate, extract, IDs, K, phi,
+                                          raffinate_chemicals, extract_chemicals)
+            if raffinate_chemicals:
+                raffinate_flows = multi_stream.imol[raffinate_chemicals]
+                for i in stages: i.raffinate.imol[raffinate_chemicals] = raffinate_flows
+            if extract_chemicals:
+                extract_flows = multi_stream.imol[extract_chemicals]
+                for i in stages: i.extract.imol[extract_chemicals] = extract_flows
         else:
             lle = multi_stream.lle
             lle(multi_stream.T, top_chemical=self.carrier_chemical or feed.main_chemical)
@@ -991,7 +1171,8 @@ class MultiStageLLE:
     def multi_stage_lle_without_side_draws_iter(self, extract_flow_rates):
         self.update_multi_stage_lle_without_side_draws(extract_flow_rates)
         stages = self.stages
-        for i in stages: i.partition(self.partition_data)
+        for i in stages: i.balance_raffinate_flows()
+        for i in stages: i.partition(self.partition_data, update=False)
         K = np.transpose([i.K for i in stages]) 
         phi = np.array([i.phi for i in stages])
         index = self.index
@@ -1084,6 +1265,8 @@ def flow_rates_for_multi_stage_extration_without_side_draws(
         Extract flow rates with stages by row and components by column.
 
     """
+    phase_fractions[phase_fractions < 1e-16] = 1e-16
+    phase_fractions[phase_fractions > 1. - 1e-16] = 1. - 1e-16
     phase_ratios = phase_fractions / (1. - phase_fractions)
     N_chemicals = feed.size
     extract_flow_rates = np.zeros((N_stages, N_chemicals))
