@@ -9,8 +9,11 @@
 """
 from ..registry import Registry
 
-__all__ = ('registered',)
+__all__ = ('registered', 'unregistered')
 
+def unregistered(cls):
+    cls._register = _pretend_to_register
+    return cls
 
 def registered(ticket_name):
     return lambda cls: _registered(cls, ticket_name)
@@ -18,11 +21,10 @@ def registered(ticket_name):
 def _registered(cls, ticket_name):
     cls.registry = Registry()
     cls.ticket_name = ticket_name
-    cls.ticket_number = 0
+    cls.ticket_numbers = {}
     cls.unregistered_ticket_number = 0
     cls._take_unregistered_ticket = _take_unregistered_ticket
     cls._take_ticket = _take_ticket
-    cls._unregister = _unregister
     cls._register = _register
     cls.ID = ID
     cls.__repr__ = __repr__
@@ -31,8 +33,14 @@ def _registered(cls, ticket_name):
 
 @classmethod
 def _take_ticket(cls):
-    cls.ticket_number += 1
-    return cls.ticket_name + str(cls.ticket_number)
+    ticket_numbers = cls.ticket_numbers
+    ticket_name = cls.ticket_name 
+    if ticket_name not in ticket_numbers:
+        ticket_numbers[ticket_name] = n = 1
+    else:
+        ticket_numbers[ticket_name] = n = ticket_numbers[ticket_name] + 1
+    return ticket_name + str(n)
+
 @classmethod
 def _take_unregistered_ticket(cls):
     if cls.unregistered_ticket_number > 100:
@@ -41,20 +49,24 @@ def _take_unregistered_ticket(cls):
         cls.unregistered_ticket_number += 1
     return cls.ticket_name + '.' + str(cls.unregistered_ticket_number)
 
-def _unregister(self):
-    try: delattr(self.registry, self._ID)
-    except: pass
-    
 def _register(self, ID):
-    if ID:
-        self.registry[ID] = self
-        self._ID = ID
-    elif ID == "":
-        self._ID = ID = self._take_ticket()
-        self.registry.__dict__[ID] = self
+    replace_ticket_number = isinstance(ID, int)
+    if replace_ticket_number: self.__class__.ticket_number = ID
+    if ID == "" or replace_ticket_number: 
+        registry = self.registry
+        data = registry.data
+        while True:
+            ID = self._take_ticket()
+            if not ID in data: break
+        registry.register(ID, self)
+    elif ID:
+        self.registry.register_safely(ID, self) 
     else:
         self._ID = self._take_unregistered_ticket()
-        
+
+def _pretend_to_register(self, ID):
+    self._ID = ID
+
 @property
 def ID(self):
     """Unique identification (str). If set as '', it will choose a default ID."""
@@ -62,7 +74,6 @@ def ID(self):
 
 @ID.setter
 def ID(self, ID):
-    self._unregister()
     self._register(ID)
     
 def __repr__(self):
